# mapping.py
# AE3M33UI - Artificial Intellignece course, FEE CTU in Prague
#
# Module containing mapping functions.

import numpy as np
from itertools import combinations_with_replacement

class PolynomialMapping:
    
    def __init__(self, max_deg=2):
        self.max_deg = max_deg
        
    def fit(self, X, y=None):
        return self
        
    def transform(self, X, y=None):
        """Map the input data x into space of polynomials."""
        if self.max_deg == 0:
            return np.ones((X.shape[0],1))
        Xm = X.copy()
        for deg in range(2,self.max_deg+1):
            Xm = np.hstack((Xm, X**deg))
        return Xm

class FullPolynomialMapping:
    
    def __init__(self, max_deg=2):
        self.max_deg = max_deg
        
    def fit(self, X, y=None):
        return self
        
    def transform(self, X, y=None, max_deg=2):
        """Map X to fully polynomial space, including all crossproducts."""
        pX = X.copy()
        n_ex = X.shape[0]
        n_col = X.shape[1]
        for deg in range(2,self.max_deg+1):
            combs = combinations_with_replacement(range(n_col), deg)
            for comb in combs:
                vec = np.ones((n_ex,), dtype=np.float)
                for col in comb:
                    vec *= X[:,col]
                vec = vec[:,np.newaxis]
                pX = np.hstack((pX,vec))
        return pX
    

